/*************************************************************************
 * GBA sound hardware definitions
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 ************************************************************************/

#ifndef	_GBA_SOUND_H
#define	_GBA_SOUND_H

#include "types.h"


// sound hardware defines
// NOTE: for now, these are copied from the "official" gba.h that most demo people use... I don't know
// who to give credit for it
#define pREG_SOUND1CNT_L	(u16 *)0x04000060	// sound 1 sweep control
#define pREG_SOUND1CNT_H	(u16 *)0x04000062	// sound 1 length, wave duty and envelope control
#define pREG_SOUND1CNT_X	(u16 *)0x04000064	// sound 1 frequency, reset and loop control
#define pREG_SOUND2CNT_L	(u16 *)0x04000068	// sound 2 length, wave duty and envelope control
#define pREG_SOUND2CNT_H	(u16 *)0x0400006C	// sound 2 frequency, reset and loop control
#define pREG_SOUND3CNT_L	(u16 *)0x04000070	// sound 3 enable and wave ram bank control
#define pREG_SOUND3CNT_H	(u16 *)0x04000072	// sound 3 sound length and output level control
#define pREG_SOUND3CNT_X	(u16 *)0x04000074	// sound 3 frequency, reset and loop control
#define pREG_SOUND4CNT_L	(u16 *)0x04000078	// sound 4 length, output level and envelope control
#define pREG_SOUND4CNT_H	(u16 *)0x0400007C	// sound 4 noise parameters, reset and loop control
#define pREG_SOUNDCNT_L		(u16 *)0x04000080	// sound 1-4 output level and stereo control
#define pREG_SOUNDCNT_H		(u16 *)0x04000082	// direct sound control and sound 1-4 output ratio
#define pREG_SOUNDCNT_X		(u32 *)0x04000084	// master sound enable and sound 1-4 play status
#define pREG_SOUNDBIAS		(u32 *)0x04000088	// sound bias and amplitude resolution control
#define pREG_WAVE_RAM0_L	(u16 *)0x04000090	// sound 3 samples 0-3
#define pREG_WAVE_RAM0_H	(u16 *)0x04000092	// sound 3 samples 4-7
#define pREG_WAVE_RAM1_L	(u16 *)0x04000094	// sound 3 samples 8-11
#define pREG_WAVE_RAM1_H	(u16 *)0x04000096	// sound 3 samples 12-15
#define pREG_WAVE_RAM2_L	(u16 *)0x04000098	// sound 3 samples 16-19
#define pREG_WAVE_RAM2_H	(u16 *)0x0400009A	// sound 3 samples 20-23
#define pREG_WAVE_RAM3_L	(u16 *)0x0400009C	// sound 3 samples 23-27
#define pREG_WAVE_RAM3_H	(u16 *)0x0400009E	// sound 3 samples 28-31

#define REG_SOUND1CNT_L		*pREG_SOUND1CNT_L	// sound 1 sweep control
#define REG_SOUND1CNT_H		*pREG_SOUND1CNT_H	// sound 1 length, wave duty and envelope control
#define REG_SOUND1CNT_X		*pREG_SOUND1CNT_X	// sound 1 frequency, reset and loop control
#define REG_SOUND2CNT_L		*pREG_SOUND2CNT_L	// sound 2 length, wave duty and envelope control
#define REG_SOUND2CNT_H		*pREG_SOUND2CNT_H	// sound 2 frequency, reset and loop control
#define REG_SOUND3CNT_L		*pREG_SOUND3CNT_L	// sound 3 enable and wave ram bank control
#define REG_SOUND3CNT_H		*pREG_SOUND3CNT_H	// sound 3 sound length and output level control
#define REG_SOUND3CNT_X		*pREG_SOUND3CNT_X	// sound 3 frequency, reset and loop control
#define REG_SOUND4CNT_L		*pREG_SOUND4CNT_L	// sound 4 length, output level and envelope control
#define REG_SOUND4CNT_H		*pREG_SOUND4CNT_H	// sound 4 noise parameters, reset and loop control
#define REG_SOUNDCNT_L		*pREG_SOUNDCNT_L	// sound 1-4 output level and stereo control
#define REG_SOUNDCNT_H		*pREG_SOUNDCNT_H	// direct sound control and sound 1-4 output ratio
#define REG_SOUNDCNT_X		*pREG_SOUNDCNT_X	// master sound enable and sound 1-4 play status
#define REG_SOUNDBIAS		*pREG_SOUNDBIAS		// sound bias and amplitude resolution control
#define REG_WAVE_RAM0_L		*pREG_WAVE_RAM0_L	// sound 3 samples 0-3
#define REG_WAVE_RAM0_H		*pREG_WAVE_RAM0_H	// sound 3 samples 4-7
#define REG_WAVE_RAM1_L		*pREG_WAVE_RAM1_L	// sound 3 samples 8-11
#define REG_WAVE_RAM1_H		*pREG_WAVE_RAM1_H	// sound 3 samples 12-15
#define REG_WAVE_RAM2_L		*pREG_WAVE_RAM2_L	// sound 3 samples 16-19
#define REG_WAVE_RAM2_H		*pREG_WAVE_RAM2_H	// sound 3 samples 20-23
#define REG_WAVE_RAM3_L		*pREG_WAVE_RAM3_L	// sound 3 samples 23-27
#define REG_WAVE_RAM3_H		*pREG_WAVE_RAM3_H	// sound 3 samples 28-31

#define REG_FIFO_A			0x040000A0			// direct sound channel A samples 0-3
#define REG_FIFO_A_L		0x040000A0			// direct sound channel A samples 0-1
#define REG_FIFO_A_H		0x040000A2			// direct sound channel A samples 2-3
#define REG_FIFO_B			0x040000A4			// direct sound channel B samples 0-3
#define REG_FIFO_B_L		0x040000A4			// direct sound channel B samples 0-1
#define REG_FIFO_B_H		0x040000A6			// direct sound channel B samples 2-3

// NOTE: this comes from the demo by Chris Strickland, from the GameDev.net audio tutorial
// some sound bit defines
#define SND_ENABLED           0x00000080
#define SND_OUTPUT_RATIO_25   0x0000
#define SND_OUTPUT_RATIO_50   0x0001
#define SND_OUTPUT_RATIO_100  0x0002
#define DSA_OUTPUT_RATIO_50   0x0000
#define DSA_OUTPUT_RATIO_100  0x0004
#define DSA_OUTPUT_TO_RIGHT   0x0100
#define DSA_OUTPUT_TO_LEFT    0x0200
#define DSA_OUTPUT_TO_BOTH    0x0300
#define DSA_TIMER0            0x0000
#define DSA_TIMER1            0x0400
#define DSA_FIFO_RESET        0x0800
#define DSB_OUTPUT_RATIO_50   0x0000
#define DSB_OUTPUT_RATIO_100  0x0008
#define DSB_OUTPUT_TO_RIGHT   0x1000
#define DSB_OUTPUT_TO_LEFT    0x2000
#define DSB_OUTPUT_TO_BOTH    0x3000
#define DSB_TIMER0            0x0000
#define DSB_TIMER1            0x4000
#define DSB_FIFO_RESET        0x8000

// default settings... 22050 Hz sound effect
#define DEFAULT_FREQUENCY             22050
#define DEFAULT_TIMER_INTERVAL        0xFD07    // (65536-(round((2^24)/FREQUENCY)))
#define DEFAULT_SAMPLES_PER_VBLANK    0x0170    // (FREQUENCY/60)

// a typedef for a sample
typedef struct _SoundEffectStruct
{
    s8		*pData;             // pointer to the raw sound data
    s32 	length;             // the size of the sample (padded to 1/60th of a second)
	s32     vblLength;			// number of vblanks this sample lasts

} SoundEffectStruct;


#endif

