/******************************************************************************
 * powerup object code
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *****************************************************************************/

#include "powerup.h"
#include "sprite.h"
#include "game.h"
#include "collide.h"


/********************************************************************
 * launch a powerup
 *
 * IN:		type = type of powerup
 * 			pPos = world position of the powerup
 * 			value = value of the powerup (for example, for a ammo powerup, it is
 * 					the amount of ammo the powerup gives if picked up)
 *
 * RETURNS:	ptr to the powerup object
 *******************************************************************/
PowerupStruct *Powerup_New (u16 type, s24f8_vect2D *pPos, u16 value)
{
u32 objID;
PowerupStruct *pPowerup;

	// grab a new object
	objID = Object_New(OBJECT_POWERUP);
	if (objID == OBJECT_ID_NULL)
	{	// error... no unused objects left
		return(NULL);
	}

	// get the pointer to it
	pPowerup = (PowerupStruct *)Object_getPtrS(objID);

	// init it
	pPowerup->powerupType = type;

	pPowerup->pos = *pPos;

	pPowerup->value = value;

	return(pPowerup);
}


/**********************************************************************
 * update a powerup object
 *********************************************************************/
u32 Powerup_Update (ObjectStruct *pObj)
{
PowerupStruct *pPowerup = (PowerupStruct *)pObj;
u16 tileNum;

	// collide us
	if (Collide_PowerupToPlayer(pPowerup))
	{
		return(1);
	}

	switch (pPowerup->powerupType)
	{
	case POWERUP_AMMO:
		tileNum = AMMO_TILENUM;
		break;
	case POWERUP_FINISH:
		tileNum = BOX_TILENUM;
		break;
	default:
		// error!
		return(1);
	}

	// draw the powerup
	Sprite_DrawQuick(s24f8_to_int(pPowerup->pos.x), s24f8_to_int(pPowerup->pos.y),
					 tileNum, POWERUP_PALETTE);

	return(0);
}


/*****************************************************************
 * give a powerup to a dude
 *
 * IN:		ptr to the dude
 * 			ptr to the powerup
 ****************************************************************/
void Powerup_GiveToDude (DudeStruct *pDude, PowerupStruct *pPowerup)
{
	switch (pPowerup->powerupType)
	{
	case POWERUP_AMMO:
		gPlayerAmmo += 5;
		break;

	case POWERUP_FINISH:
		// you won... go to the next level
		gGameLevel++;
		if (gGameLevel >= NUM_GAME_LEVELS)
		{	// you finished all the levels!
			gPlayerLives++;
			gGameLevel = 0;
		}
		gameplayNewState = GAMEPLAY_NORMAL;
		break;

	default:
		break;
	}
}

