/********************************************************************
 * text and number displaying routines
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 ********************************************************************/

#include "text.h"
#include "math.h"
#include "sprite.h"
#include "game.h"
#include "background.h"

/********************************************************************************
 * display a number at a screen position on the screen, using sprites
 *
 * IN:		screenX, screenY = screen position in pixels (0,0 = top-left corner of screen)
 * 			palNum = palette to display with (0..15)
 * 			minDigits = the minimum number of digits to display
 * 			num = value to display
 *******************************************************************************/
void TextSpr_DrawNumber (u16 screenX, u16 screenY, u16 palNum, u16 minDigits, u16 num)
{
u8 outNums[8];
u16 outCount = 0;
u16 tmp;
u16 flag = 0;

	// calculate the number as text
	if (num >= 10000)
	{
		tmp = s32_div(num, 10000);

		outNums[outCount++] = tmp;

		num -= 10000 * tmp;

		flag = 1;
	}

	if (num >= 1000)
	{
		tmp = s32_div(num, 1000);

		outNums[outCount++] = tmp;

		num -= 1000 * tmp;

		flag = 1;
	}
	else if (flag)
	{
		outNums[outCount++] = 0;
	}

	if (num >= 100)
	{
		tmp = s32_div(num, 100);

		outNums[outCount++] = tmp;

		num -= 100 * tmp;

		flag = 1;
	}
	else if (flag)
	{
		outNums[outCount++] = 0;
	}


	if (num >= 10)
	{
		tmp = s32_div(num, 10);

		outNums[outCount++] = tmp;

		num -= 10 * tmp;
	}
	else if (flag)
	{
		outNums[outCount++] = 0;
	}

	outNums[outCount++] = num;

	if (minDigits > outCount)
	{
		for (tmp=outCount; tmp<minDigits; tmp++)
		{
			Sprite_DrawQuick(screenX, screenY, (NUMS_TILENUM + 0), palNum);
			screenX += 8;
		}
	}

	for (tmp=0; tmp<outCount; tmp++)
	{
		Sprite_DrawQuick(screenX, screenY, (NUMS_TILENUM + outNums[tmp]), palNum);
		screenX += 8;
	}
}


/********************************************************************************
 * display text at a screen position on the screen, using sprites
 *
 * IN:		screenX, screenY = screen position in pixels (0,0 = top-left corner of screen)
 * 			palNum = palette to display with (0..15)
 * 			pText = text to print
 *******************************************************************************/
void TextSpr_DrawString (u16 screenX, u16 screenY, u16 palNum, u8 *pText)
{
s8 ch;
s16 tileNum;

	if (pText == NULL)
	{
		return;
	}

	while (*pText != 0)
	{
		ch = *pText++;
		tileNum = -1;

		if ((ch >= 'A') && (ch <= 'Z'))
		{	// upper case letter
			tileNum = (ch - 'A') + LETTERS_TILENUM;
		}
		else if ((ch >= 'a') && (ch <= 'z'))
		{	// lower-case letter (prints as capital letter!)
			tileNum = (ch - 'a') + LETTERS_TILENUM;
		}
		else if ((ch >= '0') && (ch <= '9'))
		{	// number
			tileNum = (ch - '0') + NUMS_TILENUM;
		}
		else if (ch == '!')
		{	// exclamation mark
			tileNum = EXCLAMATION_TILENUM;
		}
		else if (ch == '?')
		{	// question mark
			tileNum = QUESTIONMARK_TILENUM;
		}
		else if (ch == '.')
		{	// period mark
			tileNum = PERIOD_TILENUM;
		}
		else if (ch == ',')
		{	// comma mark
			tileNum = COMMA_TILENUM;
		}
		else if (ch == '-')
		{	// special character
			tileNum = DASH_TILENUM;
		}
		else if (ch == ':')
		{	// special character
			tileNum = COLON_TILENUM;
		}
		else if (ch == ' ')
		{
			screenX += 8;
		}

		if (tileNum >= 0)
		{
			Sprite_DrawQuick(screenX, screenY, tileNum, palNum);
			screenX += 8;
		}
	}
}


